//---------------------------------------------------------------------------

#ifndef MonitorBaseH
#define MonitorBaseH

#define MAXCHARLEN 250
#define MONITOREVENT "MonitorEvent"
#define MONITORDATA "MonitorData"

#include <vcl.h>
#include <Windows.hpp>
#include <Classes.hpp>
#include <SysUtils.hpp>
#include <System.hpp>

class TTagMonitorEvent {
      HANDLE FHandle;
   public:
      void __fastcall Signal(void){
         SetEvent(FHandle);
      }
      void __fastcall Reset(void){
         ResetEvent(FHandle);
      }
      bool __fastcall Wait(Integer TimeOut){
         return WaitForSingleObject(FHandle, TimeOut) == WAIT_OBJECT_0;
      }
      __fastcall TTagMonitorEvent(const AnsiString &Name, bool Manual){
         FHandle = CreateEvent(NULL, Manual, false, (Name + ".Data").c_str());
         if (FHandle == 0) Abort();
      }
      __fastcall ~TTagMonitorEvent(){
         if ( FHandle ) CloseHandle( FHandle );
      }
};//----------------------------------------------------------------------------

class TTagMonitorSharedMem {
      HANDLE FHandle;
      void *FFileView;
   public:
      __fastcall TTagMonitorSharedMem(const AnsiString &Name, int Size){
          FHandle = CreateFileMapping((HANDLE)0xFFFFFFFF, NULL, PAGE_READWRITE,
                                0, Size, Name.c_str());
          if ( FHandle == NULL ) Abort();
          FFileView = MapViewOfFile(FHandle, FILE_MAP_WRITE, 0, 0, Size);
          if ( FFileView == NULL ) Abort();
      }
      __fastcall ~TTagMonitorSharedMem(){
         if (FFileView != NULL)
            UnmapViewOfFile(FFileView);
         if ( FHandle )
            CloseHandle( FHandle );
      }
      Pointer getBuffer(){ return FFileView; }
};//----------------------------------------------------------------------------

enum TMonintorMessageType { mmt_info, mmt_warning, mmt_error };

struct TMonitorMessage {
   TMonintorMessageType type;
   int errCode;
   char text[MAXCHARLEN+1];
};//----------------------------------------------------------------------------

//+++++ TMonitorSender +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class TMonitorSender {
      TTagMonitorSharedMem *mem;
      TTagMonitorEvent *eve;
      TMonitorMessage *msg;
   protected:
      void __fastcall Send(TMonintorMessageType, const String &text, int ec=0);
   public:
      __fastcall TMonitorSender(){
         mem = new TTagMonitorSharedMem(MONITORDATA, sizeof(TMonitorMessage));
         eve = new TTagMonitorEvent(MONITOREVENT, false);
         msg = (TMonitorMessage*)mem->getBuffer();
      }//-----------------------------------------------------------------------

      __fastcall ~TMonitorSender(){
         delete mem;
         delete eve;
      }//-----------------------------------------------------------------------

      void __fastcall SendInfo(const String &text, int ec=0){
        Send(mmt_info, text, ec);
      }//-----------------------------------------------------------------------

      void __fastcall SendWarning(const String &text, int ec=0){
         Send(mmt_warning, text, ec);
      }//-----------------------------------------------------------------------

      void __fastcall SendError(const String &text, int ec=0){
         Send(mmt_error, text, ec);
      }//-----------------------------------------------------------------------

};//----------------------------------------------------------------------------

typedef void __fastcall (__closure *TMonitorMessageEvent2)(TMonintorMessageType Type, String Message, int ErrCode);

//+++++ TMonitorReceiver +++++++++++++++++++++++++++++++++++++++++++++++++++++++
class TMonitorReceiver {
      TTagMonitorSharedMem *mem;
      TTagMonitorEvent *eve;
      TMonitorMessage *msg;
      TMonitorMessageEvent2 onMessage;
   public:
      __fastcall TMonitorReceiver(TMonitorMessageEvent2 onMsg){
         mem = new TTagMonitorSharedMem(MONITORDATA, sizeof(TMonitorMessage));
         eve = new TTagMonitorEvent(MONITOREVENT, false);
         msg = (TMonitorMessage*)mem->getBuffer();
         onMessage = onMsg;
      }//-----------------------------------------------------------------------

      __fastcall ~TMonitorReceiver(){
         delete mem;
         delete eve;
      }//-----------------------------------------------------------------------

      void __fastcall Wait(long timeout);
};

#endif
